#pragma once

#define NO_BUTTON -1

uint16_t _switchStates = 0;
int _nextColToScan = 0;
int _nextRowToScan = 0;
uint8_t _colPins[4];
uint8_t _rowPins[4];

//Initialse button routines
//Buttons numbered top to bottom, left to right
//row0, col0 is button 0
//row3, col3 is button 15
void initButtons(uint8_t row0,uint8_t row1,uint8_t row2,uint8_t row3,uint8_t col0,uint8_t col1,uint8_t col2,uint8_t col3);

//Tests if any of the buttons have been pressed and released
//  returns the button that was pressed or -1
int buttonPressed();

//Tests if any of the buttons have been pressed and released
//  returns the mask of button that was pressed or 0
uint16_t maskPressed();

//------------------------------------------------------------------------------------------------------------------

//Initialse button routines
//Buttons numbered top to bottom, left to right
//row0, col0 is button 0
//row3, col3 is button 15
void initButtons(uint8_t row0,uint8_t row1,uint8_t row2,uint8_t row3,uint8_t col0,uint8_t col1,uint8_t col2,uint8_t col3)
{
  _colPins[0] = col0;
  _colPins[1] = col1;
  _colPins[2] = col2;
  _colPins[3] = col3;
  _rowPins[0] = row0;
  _rowPins[1] = row1;
  _rowPins[2] = row2;
  _rowPins[3] = row3;

  //Setup switches
  for (int i = 0; i < 4; i++)
  {
    pinMode(_colPins[i], INPUT);
    pinMode(_rowPins[i], OUTPUT);
    digitalWrite(_rowPins[i], LOW);
  }

  // Switches
  // Delay between each interrupt - eg: 1 sec ==> (16*10^6) / (1*1024) - 1 (must be <65536) = 15640
  // 1mS - 15.640
  // 5mS - 78.2
  #define SCAN_SPEED 78

  // Set up timer for background switch scanning
  cli();  //stop interrupts
  //set timer1 interrupt for flash
  TCCR1A = 0;// set entire TCCR1A register to 0
  TCCR1B = 0;// same for TCCR1B
  TCNT1  = 0;//initialize counter value to 0
  // set compare match register for 1hz increments
  OCR1A = SCAN_SPEED;
  // turn on CTC mode
  TCCR1B |= (1 << WGM12);
  // Set CS10 and CS12 bits for 1024 prescaler
  TCCR1B |= (1 << CS12) | (1 << CS10);  
  // enable timer compare interrupt
  TIMSK1 |= (1 << OCIE1A);    
  // allow interrupts
  sei();
  
}

//------------------------------------------------------------------------------------------------------------------

//timer1 interrupt for switch scanning 5mS
//Each iteration of the timer tests a single row and column. The state
//of all switches are held in switchStates.
ISR(TIMER1_COMPA_vect)
{
  //Work out which bit in switchStates reflect this row and column
  uint16_t mask = (0x0001 << _nextColToScan) << (_nextRowToScan << 2);

  //Set the row pin high
  digitalWrite(_rowPins[_nextRowToScan], HIGH);
  //Read the column pin
  if (digitalRead(_colPins[_nextColToScan]) == HIGH)
  {
    //Pressed - Set the bit that matches this button
    _switchStates = _switchStates | mask;
  }
  else
  {
    //Not pressed - Clear the bit that matches this button
    _switchStates = _switchStates & ~mask;
  }
  //Set the row pin low again
  digitalWrite(_rowPins[_nextRowToScan], LOW);

  //Update the row and column counters for next timer interrupt
  _nextColToScan = (_nextColToScan + 1) & 0x03;
  if (_nextColToScan == 0)
  {
    _nextRowToScan = (_nextRowToScan + 1) & 0x03;
  }
}

//------------------------------------------------------------------------------------------------------------------

//Tests if any of the buttons have been pressed and released
//  returns the button that was pressed or -1
int buttonPressed()
{
  int button = NO_BUTTON;   //Result store
  uint16_t mask = 0x0001;   //Mask for switchStates
  int btn = 0;              //Button/LED button number
  //Cycle through all 16 buttons
  while (btn < 16 && button == NO_BUTTON)
  {
    //Test if pressed
    if (_switchStates & mask)
    {
      //Delay for debouncing
      _delay_ms(10);
      //Test if it is still pressed otherwise it is a bounce
      if (_switchStates & mask)
      {
        //Wait until button is released
        while (_switchStates & mask)
        {
          _delay_ms(50);
        }
        //We have a valid button press
        button = btn;
      }
    }
    //Set up for next button to test
    mask = mask << 1;
    btn++;
  }
  return button;
}

//------------------------------------------------------------------------------------------------------------------

//Tests if any of the buttons have been pressed and released
//  returns mask of button that was pressed or 0
uint16_t maskPressed()
{
  uint16_t button = 0;      //Result store
  uint16_t mask = 0x0001;   //Mask for switchStates
  int btn = 0;              //Button/LED button number
  //Cycle through all 16 buttons
  while (btn < 16 && button == 0)
  {
    //Test if pressed
    if (_switchStates & mask)
    {
      //Delay for debouncing
      _delay_ms(10);
      //Test if it is still pressed otherwise it is a bounce
      if (_switchStates & mask)
      {
        //Wait until button is released
        while (_switchStates & mask)
        {
          _delay_ms(50);
        }
        //We have a valid button press
        button = mask;
      }
    }
    //Set up for next button to test
    mask = mask << 1;
    btn++;
  }
  return button;
}
